@@ -2,7 +2,7 @@
  * dpkg - main program for package management
  * update.c - options which update the ‘available’ database
  *
- * Copyright © 1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright © 1995 Ian Jackson <ijackson@chiark.greenend.org.uk>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,8 +48,10 @@ updateavailable(const char *const *argv)
     if (sourcefile) badusage(_("--%s takes no arguments"),cipaction->olong);
     break;
   case act_avreplace: case act_avmerge:
-    if (!sourcefile || argv[1])
-      badusage(_("--%s needs exactly one Packages-file argument"),
+    if (sourcefile == NULL)
+      sourcefile = "-";
+    else if (sourcefile && argv[1])
+      badusage(_("--%s takes at most one Packages-file argument"),
                cipaction->olong);
     break;
   default:
@@ -57,11 +59,15 @@ updateavailable(const char *const *argv)
   }
 
   if (!f_noact) {
-    if (access(dpkg_db_get_dir(), W_OK)) {
+    const char *dbdir = dpkg_db_get_dir();
+
+    if (access(dbdir, W_OK)) {
       if (errno != EACCES)
-        ohshite(_("unable to access dpkg status area for bulk available update"));
+        ohshite(_("unable to access dpkg database directory '%s' for bulk available update"),
+                dbdir);
       else
-        ohshit(_("bulk available update requires write access to dpkg status area"));
+        ohshit(_("required write access to dpkg database directory '%s' for bulk available update"),
+               dbdir);
     }
     modstatdb_lock();
   }
@@ -85,7 +91,9 @@ updateavailable(const char *const *argv)
     parsedb(availfile, pdb_parse_available, NULL);
 
   if (cipaction->arg_int != act_avclear)
-    count += parsedb(sourcefile, pdb_parse_available | pdb_ignoreolder, NULL);
+    count += parsedb(sourcefile,
+                     pdb_parse_available | pdb_ignoreolder | pdb_dash_is_stdin,
+                     NULL);
 
   if (!f_noact) {
     writedb(availfile, wdb_dump_available);
